"use strict";
/**
 * @license
 * Copyright Google Inc. All Rights Reserved.
 *
 * Use of this source code is governed by an MIT-style license that can be
 * found in the LICENSE file at https://angular.io/license
 */
Object.defineProperty(exports, "__esModule", { value: true });
var enums_1 = require("./enums");
function normalizeMethodName(method) {
    if (typeof method !== 'string')
        return method;
    switch (method.toUpperCase()) {
        case 'GET':
            return enums_1.RequestMethod.Get;
        case 'POST':
            return enums_1.RequestMethod.Post;
        case 'PUT':
            return enums_1.RequestMethod.Put;
        case 'DELETE':
            return enums_1.RequestMethod.Delete;
        case 'OPTIONS':
            return enums_1.RequestMethod.Options;
        case 'HEAD':
            return enums_1.RequestMethod.Head;
        case 'PATCH':
            return enums_1.RequestMethod.Patch;
    }
    throw new Error("Invalid request method. The method \"" + method + "\" is not supported.");
}
exports.normalizeMethodName = normalizeMethodName;
exports.isSuccess = function (status) { return (status >= 200 && status < 300); };
function getResponseURL(xhr) {
    if ('responseURL' in xhr) {
        return xhr.responseURL;
    }
    if (/^X-Request-URL:/m.test(xhr.getAllResponseHeaders())) {
        return xhr.getResponseHeader('X-Request-URL');
    }
    return null;
}
exports.getResponseURL = getResponseURL;
function stringToArrayBuffer8(input) {
    var view = new Uint8Array(input.length);
    for (var i = 0, strLen = input.length; i < strLen; i++) {
        view[i] = input.charCodeAt(i);
    }
    return view.buffer;
}
exports.stringToArrayBuffer8 = stringToArrayBuffer8;
function stringToArrayBuffer(input) {
    var view = new Uint16Array(input.length);
    for (var i = 0, strLen = input.length; i < strLen; i++) {
        view[i] = input.charCodeAt(i);
    }
    return view.buffer;
}
exports.stringToArrayBuffer = stringToArrayBuffer;
//# sourceMappingURL=http_utils.js.map